/* 
 SPI.cpp - SPI library for esp8266

 Copyright (c) 2015 Hristo Gochkov. All rights reserved.
 This file is part of the esp8266 core for Arduino environment.
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

// GTFT_IO.cpp

// My additions for GTFT_IO is "Help Yourself Software", no warranty, no support, no obligations.

// JZ 01.01.2017

// this is a preliminary version to check inclusion from src directory with Arduino 1.6.10

#include "GTFT_IO.h"

#if defined (ESP8266)

GTFT_IO_SPI9::GTFT_IO_SPI9(uint8_t dcPin) : _dc(dcPin)
{
}

inline void GTFT_IO_SPI9::setDataBits(uint16_t bits)
{
  const uint32_t mask = ~((SPIMMOSI << SPILMOSI) | (SPIMMISO << SPILMISO));
  bits--;
  SPI1U1 = ((SPI1U1 & mask) | ((bits << SPILMOSI) | (bits << SPILMISO)));
}

uint8_t GTFT_IO_SPI9::transfer(uint8_t data)
{
  while (SPI1CMD & SPIBUSY) {}
  // low byte first, ms bit first!; CMD : dc low -> first bit 0, DATA : dc high -> first bit high
  setDataBits(9);
  SPI1W0 = (data >> 1) | (data << 15) | (digitalRead(_dc) << 7);
  SPI1CMD |= SPIBUSY;
  while (SPI1CMD & SPIBUSY) {}
  uint16_t rv = SPI1W0;
  // return uint8_t((rv << 1) | (rv >> 15)); // most likely first try is wrong, need to find test method
  return uint8_t(rv & 0xFF); // yes, was wrong, is easy
}

uint16_t GTFT_IO_SPI9::transfer16(uint16_t data)
{
  union
  {
    uint16_t val;
    struct
    {
      uint8_t lsb;
      uint8_t msb;
    };
  } in, out;
  in.val = data;
  out.msb = transfer(in.msb);
  out.lsb = transfer(in.lsb);
  return out.val;
}

void GTFT_IO_SPI9::write(uint8_t data)
{
  while (SPI1CMD & SPIBUSY) {}
  // low byte first, ms bit first!; CMD : dc low -> first bit 0, DATA : dc high -> first bit high
  setDataBits(9);
  SPI1W0 = (data >> 1) | (data << 15) | (digitalRead(_dc) << 7);
  SPI1CMD |= SPIBUSY;
  while (SPI1CMD & SPIBUSY) {}
}

void GTFT_IO_SPI9::write16(uint16_t data)
{ // no easy optimization
  write(data >> 8);
  write(data);
}

void GTFT_IO_SPI9::write16(uint16_t data, bool msb)
{
  write16(data);
}

void GTFT_IO_SPI9::write32(uint32_t data)
{ // no easy optimization
  union
  {
    uint32_t l;
    uint8_t b[4];
  } data_;
  data_.l = data;
  write(data_.b[3]);
  write(data_.b[2]);
  write(data_.b[1]);
  write(data_.b[0]);
}

void GTFT_IO_SPI9::write32(uint32_t data, bool msb)
{
  write32(data);
}

void GTFT_IO_SPI9::writeBytes(uint8_t * data, uint32_t size)
{ // no easy optimization
  for (uint8_t i = 0; i < size; i++)
  {
    write(data[i]);
  }
  return;
}

void GTFT_IO_SPI9::writePattern(uint8_t * data, uint8_t size, uint32_t repeat)
{ // no easy optimization
  for (uint32_t r = 0; r < repeat; r++)
  {
    for (uint8_t i = 0; i < size; i++)
    {
      write(data[i]);
    }
    yield(); // avoid wdt timeout on low bitrates
  }
}
void GTFT_IO_SPI9::transferBytes(uint8_t * out, uint8_t * in, uint32_t size)
{ // no easy optimization
  for (uint32_t i = 0; i < size; i++)
  {
    out[i] = transfer(in[i]);
  }
  return;
}

#endif


