/* 
  SPI.h - SPI library for esp8266

  Copyright (c) 2015 Hristo Gochkov. All rights reserved.
  This file is part of the esp8266 core for Arduino environment.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// GTFT_IO.h

// My additions for GTFT_IO is "Help Yourself Software", no warranty, no support, no obligations.

// JZ 01.01.2017

// this is a preliminary version to check inclusion from src directory with Arduino 1.6.10

#ifndef _GTFT_IO_H_
#define _GTFT_IO_H_

#include <SPI.h>

class GTFT_IO_Base : public SPIClass
{
public:
  GTFT_IO_Base(){};
  virtual void begin() {SPIClass::begin();};
  virtual void end() {SPIClass::begin();};
  virtual void setHwCs(bool use) {SPIClass::begin();};
  virtual void setBitOrder(uint8_t bitOrder) {SPIClass::begin();};
  virtual void setDataMode(uint8_t dataMode) {SPIClass::begin();};
  virtual void setFrequency(uint32_t freq) {SPIClass::begin();};
  virtual void setClockDivider(uint32_t clockDiv) {SPIClass::begin();};
  virtual void beginTransaction(SPISettings settings) {SPIClass::begin();};
  virtual uint8_t transfer(uint8_t data) {SPIClass::begin();};
  virtual uint16_t transfer16(uint16_t data) {SPIClass::begin();};
  virtual void write(uint8_t data) {SPIClass::begin();};
  virtual void write16(uint16_t data) {SPIClass::begin();};
  virtual void write16(uint16_t data, bool msb) {SPIClass::begin();};
  virtual void write32(uint32_t data) {SPIClass::begin();};
  virtual void write32(uint32_t data, bool msb) {SPIClass::begin();};
  virtual void writeBytes(uint8_t * data, uint32_t size) {SPIClass::begin();};
  virtual void writePattern(uint8_t * data, uint8_t size, uint32_t repeat) {SPIClass::begin();};
  virtual void transferBytes(uint8_t * out, uint8_t * in, uint32_t size) {SPIClass::begin();};
  virtual void endTransaction(void) {SPIClass::begin();};
};

#if defined (ESP8266)

class GTFT_IO_SPI9 : public  GTFT_IO_Base
{
public:
  GTFT_IO_SPI9(uint8_t dcPin);
  uint8_t transfer(uint8_t data);
  uint16_t transfer16(uint16_t data);
  void write(uint8_t data);
  void write16(uint16_t data);
  void write16(uint16_t data, bool msb);
  void write32(uint32_t data);
  void write32(uint32_t data, bool msb);
  void writeBytes(uint8_t * data, uint32_t size);
  void writePattern(uint8_t * data, uint8_t size, uint32_t repeat);
  void transferBytes(uint8_t * out, uint8_t * in, uint32_t size);
  private:
    uint8_t _dc;
    inline void setDataBits(uint16_t bits);
};

#endif

#endif
